import Ember from 'ember';

var IndexController = Ember.ArrayController.extend({

  // Query params are sort, order, filter, and filterValue
  // sync with the IndexRoute
  queryParams: ['sort', 'order', 'filter', 'filterValue'],
  sort:        null,
  order:       null,
  filter:      null,
  filterValue: null, 
    
  actions: {

    // Toggle refinements from button.
    toggleRefinements: function() {
      this.set('showRefinements', !this.get('showRefinements'));
    },
    
    // Refine the search based on the filters.
    refineSearchAction: function() {
      this.refineSearch();
    }
  },

  // Sort by options exposed in the drop down.
  sortOptions: [
    {
      label: 'Default',
      value: 'id'
    },
    {
      label: 'Title',
      value: 'title'
    },
    {
      label: 'Author',
      value: 'author'
    },
    {
      label: 'Publication Year',
      value: 'pubYear'
    },
    {
      label: 'Purchase Date',
      value: 'purchaseDate'
    },
    {
      label: 'Rating',
      value: 'rating'
    },
    {
      label: 'Read',
      value: 'wasRead'
    }
  ],
  
  // Ordering of the sort exposed in a drop down.
  orderOptions: [
    {
      label: 'Ascending',
      value: 'ASC'
    },
    {
      label: 'Descending',
      value: 'DESC'
    }
  ],

  // Filter by options exposed in the drop down.
  filterOptions: [
    {
      label: 'None',
      value: ''
    },
    {
      label: 'Title',
      value: 'title'
    },
    {
      label: 'Author',
      value: 'author'
    },
    {
      label: 'Publication Year',
      value: 'pubYear'
    },
    {
      label: 'Purchase Date',
      value: 'purchaseDate'
    },
    {
      label: 'Rating',
      value: 'rating'
    },
    {
      label: 'Read',
      value: 'read'
    },
    {
      label: 'Unread',
      value: 'unread'
    }
  ],
  
  // Selected sort by option.
  selectedSortOption: null,

  // Selected order option.
  selectedOrderOption: null,

  // Selected filter by option.
  selectedFilterOption: null,

  // Selecte filter value.
  selectedFilterValue: null,
  
  // Show refinements or not
  showRefinements: false,
  
  // When true, refine button is disabled
  refineButtonDisabled: true,
  
  // If true, then show the filter value field
  showFilterValue: function() {
    var filterOption = this.get('selectedFilterOption');
    if (filterOption) {
      if (filterOption.value !== this.get('filterOptions.0.value') &&
          filterOption.value !== 'read' &&
          filterOption.value !== 'unread') {
            return true;
          }
    }
    return false;
  }.property('selectedFilterOption'),

  // If true, then show the filter value field
  showOrder: function() {
    var sortOption = this.get('selectedSortOption');
    return (sortOption && sortOption.value !== this.get('sortOptions.0.value'));
  }.property('selectedSortOption'),

  // Show when showRefinements is set to false, then clear the search params
  refinementVisibilityDidChange: function() {
    if (!this.get('showRefinements')) {
      this.clearSearch();
    }
  }.observes('showRefinements'),
  
  sortAndFilterOptionsDidChange: function() {
    var sortOption   = this.get('selectedSortOption'),
        filterOption = this.get('selectedFilterOption');

    this.set('refineButtonDisabled', 
            (((sortOption && sortOption.value   === this.get('sortOptions.0.value')) &&
            (filterOption && filterOption.value === this.get('filterOptions.0.value'))))); 

  }.observes('selectedSortOption', 'selectedFilterOption'),
  
  
  // Refine the search based on the selected filters
  refineSearch: function() {
    var selectedSortOption   = this.get('selectedSortOption.value'),
        selectedOrderOption  = this.get('selectedOrderOption.value'),
        selectedFilterOption = this.get('selectedFilterOption.value'),
        selectedFilterValue  = this.get('selectedFilterValue');
    
    if (selectedSortOption !== this.get('sortOptions.0.value')) {
      this.set('sort',  selectedSortOption)
          .set('order', selectedOrderOption ? selectedOrderOption : this.get('orderOptions.0.value'));
    } else {
      this.set('sort',  null)
          .set('order', null);
    }
    
    if (selectedFilterOption !== this.get('filterOptions.0.value')) {
      
      if (selectedFilterOption === 'unread' || selectedFilterOption === 'read') {
        this.set('filter',      'wasRead')
            .set('filterValue', (selectedFilterOption === 'unread') ? false : true);
      } else {
        this.set('filter', selectedFilterOption);
        
        if (selectedFilterValue !== null) {
          this.set('filterValue', selectedFilterValue);
        }
      }
    } else {
      this.set('filter',      null)
          .set('filterValue', null);
    }
  },
  
  // Null out all filters
  clearSearch: function() {
    this.set('sort',        null)
        .set('order',       null)
        .set('filter',      null)
        .set('filterValue', null)
        .set('selectedSortOption',   this.get('sortOptions.0'))
        .set('selectedFilterOption', this.get('filterOptions.0'))
        .set('selectedFilterValue',    null);
  },

  // Make sure that the values are initialized correctly when entering
  // the route.
  syncFiltersOnInit: function() {
    var hasValidFilters = false,
        sortOptions     = this.get('sortOptions'),
        orderOptions    = this.get('orderOptions'),
        filterOptions   = this.get('filterOptions'),
        sort            = this.get('sort'),
        order           = this.get('order'),
        filter          = this.get('filter'),
        filterValue     = this.get('filterValue'),
        self            = this;

    sortOptions.forEach(function(option) {
      if (option && option.value === sort) {
        hasValidFilters = true;
        self.set('selectedSortOption', option);
      }
    });

    orderOptions.forEach(function(option) {
      if (option && option.value === order) {
        hasValidFilters = true;
        self.set('selectedOrderOption', option);
      }
    });
    
    
    if (filter === 'wasRead') {
      if (filterValue === 'true') {
        this.set('selectedFilterOption', filterOptions[6]);
      } else {
        this.set('selectedFilterOption', filterOptions[7]);
      }
      hasValidFilters = true;
    } else {
      filterOptions.forEach(function(option) {
        if (option && option.value === filter) {
          hasValidFilters = true;
          self.set('selectedFilterOption', option);
        }
      });
      if (filterValue !== null) {
        this.set('selectedFilterValue', filterValue);
      }
    }
    
        
    
    if (!hasValidFilters) {
      this.set('selectedSortOption',   sortOptions[0])
          .set('selectedOrderOption',  orderOptions[0])
          .set('selectedFilterOption', filterOptions[0])
          .set('selectedFilterValue',  null);
    } else {
      this.set('showRefinements', true);
    }
    
  },
  
  init: function() {
    this._super();
    
    // Run this in the next run loop to make sure that everything has been
    // propagated to the controller.
    Ember.run.once(this, 'syncFiltersOnInit');
  }
  
  
  
});


export default IndexController;
